--[[ List ]]
--[[ My ]]
--[[ MobDataAccesser ]]
--[[ PlayerDataAccesser ]]
--[[ ExceptDataAccesser ]]
Actor = {}
ActorStatus = {}


--[[ Public ]]
function Actor.New(ActorID)
	local Act = {
		ID					= -1,
		Delay			= 0,
		DangerLevel	= 0,
		Priority			= 0,
		IFF				= IFF_UNKNOWN,
		Pos				= { X=0, Y=0 },
		PrevPos		= { X=0, Y=0 },
		Motion			= 0,
		Target			= 0,
		EnemyList		= List.New(),
		Status			= ActorStatus.New(),
	}
	
	if ActorID ~= nil then
		Act.ID			= ActorID
		Act.Status		= ActorStatus.GetActorStatus(ActorID)
		Act				= Actor.Update(Act)
	end
	
	return Act
end

--[[ Public ]]
function Actor.GetIFF(ActorID)
	if Actor.IsOwner(ActorID) then
		return IFF_OWNER
		
	elseif Actor.IsMe(ActorID) then
		return IFF_ME
		
	elseif Actor.IsEnemy(ActorID) then
		return IFF_ENEMY
		
	elseif Actor.IsFriend(ActorID) then
		return IFF_FRIEND
		
	else
		return IFF_UNKNOWN
	end
end

--[[ Public ]]
function Actor.Update(ActorData)
	ActorData.PrevPos.X	= ActorData.Pos.X
	ActorData.PrevPos.Y	= ActorData.Pos.Y
	
	ActorData.Pos.X, ActorData.Pos.Y = GetV(V_POSITION, ActorData.ID)

	ActorData.Motion = GetV(V_MOTION, ActorData.ID)
	ActorData.Target = GetV(V_TARGET, ActorData.ID)
	
	ActorData.IFF	= Actor.GetIFF(ActorData.ID)
	
	return ActorData
end


---------------------


--[[ Public ]]
function ActorStatus.New()
	--MobData.lua Q
	local st = {
		Name	= "H",
		Lv			= 0,
		MaxHP	= 0,
		MaxSP	= 0,
		HP		= 0,
		SP		= 0,
		
		Str		= 0,
		Dex		= 0,
		Int		= 0,
		Vit		= 0,
		Luk		= 0,
		
		Atk		= 0,
		Matk		= 0,
		Hit		= 0,
		Critical	= 0,
		Def		= 0,
		Mdef		= 0,
		Flee		= 0,
		Aspd		= 0,
		
		AtkMax	= 0,
		AtkMin	= 0,
		DefDiv	= 0,
		DefSub	= 0,
		MatkMax= 0,
		MatkMin= 0,
		MdefDiv	= 0,
		MdefSub= 0,
		Range	= 0,
		
		NeedHit	= 0,
		NeedFlee=0,
		Race		= 0,
		Element = 0,
		Size		= 0,
		Speed	= 0,
		
		BaseXP	= 0,
		JobXP	= 0,
		
		Loot		= false,
		Link		= false,
		Spel		= false,
		Active	= false,
		Splash	= false,
		Long		= false,		
	}
	
	return st
end

--[[ Public ]]
function ActorStatus.GetActorStatus(ActorID)
	if My.SearchStateEnable then
		if Actor.IsMe(ActorID) then
			return My.Status
			
		elseif Actor.IsOwner(ActorID) then
			return My.Owner.Status
			
		elseif Actor.IsMonster(ActorID) then
			return MobDataAccesser.GetMobData(ActorID)
			
		elseif Actor.IsFriend(ActorID) then
			return PlayerDataAccesser.GetPlayerData(ActorID)
			
		elseif Actor.IsEnemy(ActorID) then
			
		else 
			return ActorStatus.New()
		end
		
	else
		return ActorStatus.New()
	end 
end


---------------------


--[[ Public ]]
function Actor.IsMonster(ActorID)
	if IsTrue(IsMonster(ActorID)) then
		return true
	else
		return false
	end
end

--[[ Public ]]
function Actor.IsMe(ActorID)
	if My.ID == ActorID then
		return true
	else
		return false
	end
end

--[[ Public ]]
function Actor.IsEnemy(ActorID)
	if Actor.IsMonster(ActorID) then
		if Actor.IsHomun(ActorID) or Actor.IsPlant(ActorID) then
			return not Actor.IsFriend(ActorID)	--FriendȂEnemył͂Ȃ̂ falseԂ
		else
			return true
		end
		
	elseif Actor.IsPlayer(ActorID) then
		--if not Actor.IsFriend(ActorID) and Actor.IsGvG then
		if Actor.IsFriend(ActorID) then			--FriendȂOwner^[QbgĂĂGł͂Ȃ
			return false
		else
			local OwnerTarget = GetV(V_TARGET, My.Owner.ID)
			
			if ActorID == OwnerTarget then
				return true
			else
				return false
			end
		end
	end
end

--[[ Public ]]
function Actor.IsPlayer(ActorID)
	if (ActorID < 100000) then
		return false
	end

	local ActorType= GetV(V_HOMUNTYPE, ActorID)

	if (0 <= ActorType and ActorType <= 23) then
		return true
		
	elseif (4001 <= ActorType and ActorType <= 4045) then
		return true
		
	else 
		return false
	end
end

--[[ Public ]]
function Actor.IsParty(ActorID)
	local PartyData = Party.GetParty(ActorID)

	if PartyData==nil then
		return false
	else
		return true
	end
end

--[[ Public ]]
function Actor.IsFriend(ActorID)
	local FriendData = PlayerDataAccesser.GetFriend(ActorID)

	if FriendData==nil then
		return false
	else
		return true
	end
end

--[[ Public ]]
function Actor.IsExcept(ActorID)
	local ExceptData = ExceptDataAccesser.GetExcept(ActorID)
	
	if ExceptData==nil then
		return false
	else
		return true
	end
end

--[[ Public ]]
function Actor.IsOwner(ActorID)
	if ActorID == My.Owner.ID then
		return true
	else
		return false
	end
end

--[[ Public ]]
function Actor.IsHomun(ActorID)	
	if (ActorID<0) or (65535<ActorID ) then
		return false
	end
	
	local ActorType = GetV( V_HOMUNTYPE, ActorID)
	
	if 1<= ActorType and ActorType <= 16 then
		if Actor.IsMonster(ActorID) then
			return false
		else
			return true
		end
	else
		return true
	end
end

--[[ Public ]]
function Actor.IsLif(ActorID)
	local Type = GetV(V_HOMUNTYPE, ActorID)
	
	if Type==LIF or Type==LIF2 or Type==LIF_H or Type==LIF_H2 then
		return true
	else
		return
	end
end

--[[ Public ]]
function Actor.IsAmistr(ActorID)
	local Type = GetV(V_HOMUNTYPE, ActorID)
	
	if Type==AMISTR or Type==AMISTR2 or Type==AMISTR_H or Type==AMISTR_H2 then
		return true
	else
		return
	end
end

--[[ Public ]]
function Actor.IsFilir(ActorID)
	local Type = GetV(V_HOMUNTYPE, ActorID)
	
	if Type==FILIR or Type==FILIR2 or Type==FILIR_H or Type==FILIR_H2 then
		return true
	else
		return
	end
end

--[[ Public ]]
function Actor.IsVanilmirth(ActorID)
	local Type = GetV(V_HOMUNTYPE, ActorID)
	
	if Type==VANILMIRTH or Type==VANILMIRTH2 or Type==VANILMIRTH_H or Type==VANILMIRTH_H2 then
		return true
	else
		return
	end
end


--[[ Public ]]
function Actor.IsPlant(ActorID)
	if Actor.IsPlayer(ActorID) then -- idvC[͈̔
		return false
	end

	local ActorType = GetV(V_HOMUNTYPE, ActorID)
	local PlantID = { [1555]=true, [1575]=true, [1579]=true, [1589]=true, [1590]=true }
	
	if PlantID[ActorType] then
		return true
	else
		return false
	end
end
