Tracer = {}


--[[ Private Parameter ]]
local TraceIndent	= -1
local TableIndent	= 1
local IndentLevel	= 2
local TraceList		= {first=-1, last=0}


--[[ Public ]]
function Trace( Label, Data, Level )
	if TRACE_ON then
		local LabelType = type(Label)
		local DataType  = type(Data)
		
		if Level == nil then 
			Level = TraceIndent+1
		else
			--Level = Level + TraceIndent
		end
		
		if LabelType=="string"then
			if Level <= DEBUG_LEVEL then
				List.PushRight(TraceList, Label)
			end
		end
		
		if Label==TraceList[TraceList.last] then
			if Data=="End" or Data=="" or Data==nil then
				List.PopRight(TraceList)
			end
			
			if LabelType == "number" then
				Label = tostring(Label)
				LabelType="string"
			end
			if DataType  == "number" then
				Data=tostring(Data)
				DataType ="string"
			end
			
			if LabelType=="string" and DataType=="nil" then
				Tracer.Str(Label)
				
			elseif LabelType=="string" and DataType=="string" then
				Tracer.Str(Label, Data)
				
			elseif LabelType=="string" and DataType=="table" then
				Tracer.Str(Label)
				Tracer.Tbl(Data)
				
			elseif LabelType=="table" and DataType=="nil" then
				Tracer.Tbl(Label)
					
			elseif LabelType=="table"  and DataType=="string" then
				Tracer.Tbl(Label)
				Tracer.Str(Data)
				
			elseif LabelType=="table"  and DataType=="table" then
				Tracer.Tbl(Label)
				Tracer.Tbl(Data)
				
			else
				assert(Label)
			end
		end
		
		PrevLevel = Level
	end
end

--[[ Public ]]
function Tracer.Str( ... )
	-- String|Table,   String|Table|"Start"|"End",   TraceLevel|nil
	
	local Flag		="      "
	local StartFlg	=(TraceIndent+1) ..	" "
	local EndFlg	=TraceIndent ..		" "
	
	if arg[2] == "Start" then
		TraceIndent = TraceIndent + 1 
		Flag = StartFlg
		
	elseif arg[2] == "End" then
		Flag = EndFlg
	end

	
	local IndentString = "  . "
	
	for i = 0 , TraceIndent * IndentLevel do
		IndentString = IndentString.."  "
	end
	
	TraceAI(TICK_TIME.." "..IndentString..Flag..arg[1])
	
	if arg[2] == "End" then
		TraceIndent = TraceIndent - 1
	end
end

--[[ Public ]]
function Tracer.Tbl(Table)
	for i, val in pairs( Table ) do
		if not( type(val) == "function") then
		
			TableIndent = (TableIndent + 1 )
			
			if Ask.IsTable( val ) then
				Tracer.Tbl(val)
			else
				if (TableIndent==0) or (TraceIndent+TableIndent-1==0) then
					Tracer.Str(i.." : "..val)
				else
					IndentString = ""
					for i = 0 , TableIndent do
						IndentString = IndentString.."  "
					end
					Tracer.Str(IndentString.."t"..TableIndent.." "..i.." : "..val)
				end
			end
			
			TableIndent = (TableIndent - 1 )
			
			if TableIndent == 1 then
				Trace("")
			end
		end
	end
end

--[[ Public ]]
function Tracer.Index(list)	
	if unpack(list)==nil then
		TraceAI("    list item is nil")
	else
		for i, val in list do
			TraceAI("   " .. i .. "  " .. val )
		end
	end
end
-----------------------
